@@ -8,6 +8,7 @@ import (
 	"path/filepath"
 	"strings"
 
+	"github.com/hashicorp/go-version"
 	"github.com/mattn/go-shellwords"
 )
 
@@ -56,8 +57,8 @@ func NewPlan(b []byte) *Plan {
 // As a result, the interface is opinionated and less flexible. For running arbitrary terraform commands
 // you can use Run(), which is a low-level generic method.
 type TerraformCLI interface {
-	// Version returns a version number of Terraform.
-	Version(ctx context.Context) (string, error)
+	// Version returns a Terraform version.
+	Version(ctx context.Context) (*version.Version, error)
 
 	// Init initializes the current work directory.
 	Init(ctx context.Context, opts ...string) error
@@ -77,6 +78,10 @@ type TerraformCLI interface {
 	// If a state is given, use it for the input state.
 	Import(ctx context.Context, state *State, address string, id string, opts ...string) (*State, error)
 
+	// Providers shows a tree of modules in the referenced configuration annotated with
+	// their provider requirements.
+	Providers(ctx context.Context) (string, error)
+
 	// StateList shows a list of resources.
 	// If a state is given, use it for the input state.
 	StateList(ctx context.Context, state *State, addresses []string, opts ...string) ([]string, error)
@@ -96,6 +101,13 @@ type TerraformCLI interface {
 	// because the terraform state rm command doesn't have -state-out option.
 	StateRm(ctx context.Context, state *State, addresses []string, opts ...string) (*State, error)
 
+	// StateReplaceProvider replaces a provider from source to destination address.
+	// If a state argument is given, use it for the input state.
+	// It returns the given state.
+	// Unlike other state subcommands, the terraform state replace-provider
+	// command doesn't support a -state-out option; it only supports the -state option.
+	StateReplaceProvider(ctx context.Context, state *State, source string, destination string, opts ...string) (*State, error)
+
 	// StatePush pushes a given State to remote.
 	StatePush(ctx context.Context, state *State, opts ...string) error
 
@@ -124,10 +136,14 @@ type TerraformCLI interface {
 	// so we need to switch the backend to local for temporary state operations.
 	// The filename argument must meet constraints for override file.
 	// (e.g.) _tfexec_override.tf
-	OverrideBackendToLocal(ctx context.Context, filename string, workspace string, isBackendTerraformCloud bool, backendConfig []string) (func(), error)
+	OverrideBackendToLocal(ctx context.Context, filename string, workspace string, isBackendTerraformCloud bool, backendConfig []string, supportsStateReplaceProvider bool) (func(), error)
 
 	// PlanHasChange is a helper method which runs plan and return true if the plan has change.
 	PlanHasChange(ctx context.Context, state *State, opts ...string) (bool, error)
+
+	// SupportsStateReplaceProvider is a helper method used to determine whether or
+	// not the terraform version supports `state replace-provider`.
+	SupportsStateReplaceProvider(ctx context.Context) (bool, version.Constraints, error)
 }
 
 // terraformCLI implements the TerraformCLI interface.
@@ -197,7 +213,7 @@ func (c *terraformCLI) SetExecPath(execPath string) {
 // The filename argument must meet constraints in order to override the file.
 // (e.g.) _tfexec_override.tf
 func (c *terraformCLI) OverrideBackendToLocal(ctx context.Context, filename string,
-	workspace string, isBackendTerraformCloud bool, backendConfig []string) (func(), error) {
+	workspace string, isBackendTerraformCloud bool, backendConfig []string, supportsStateReplaceProvider bool) (func(), error) {
 	// create local backend override file.
 	path := filepath.Join(c.Dir(), filename)
 	contents := `
@@ -261,12 +277,16 @@ terraform {
 		if !isBackendTerraformCloud {
 			args = append(args, "-reconfigure")
 		}
-		err = c.Init(ctx, args...)
 
+		err = c.Init(ctx, args...)
 		if err != nil {
-			// we cannot return error here.
-			log.Printf("[ERROR] [executor@%s] failed to switch back to remote: %s\n", c.Dir(), err)
-			log.Printf("[ERROR] [executor@%s] please re-run terraform init -reconfigure\n", c.Dir())
+			if supportsStateReplaceProvider && strings.Contains(err.Error(), AcceptableLegacyStateInitError) {
+				log.Printf("[INFO] [migrator@%s] ignoring error '%s'; the error is expected when using Terraform with a legacy Terraform state\n", c.Dir(), AcceptableLegacyStateInitError)
+			} else {
+				// we cannot return error here.
+				log.Printf("[ERROR] [executor@%s] failed to switch back to remote: %s\n", c.Dir(), err)
+				log.Printf("[ERROR] [executor@%s] please re-run terraform init -reconfigure\n", c.Dir())
+			}
 		}
 	}
 
